<?php

namespace Nest\Types\Definitions;

use Nest\Types\Exceptions\InvalidSerializedValueException;
use Nest\Types\Exceptions\InvalidStoredValueException;
use Carbon\Carbon;
use DateTimeInterface;
use Override;

class CarbonType extends DateTimeType
{
	/**
	 * @inheritDoc
	 * @throws InvalidSerializedValueException
	 */
	#[Override] public function deserialize(float|object|int|array|string $value): Carbon
	{
		if (!is_string($value))
			throw new InvalidSerializedValueException($this, $value);

		return Carbon::createFromFormat(DateTimeInterface::ATOM, $value);
	}

	/**
	 * @inheritDoc
	 * @throws InvalidStoredValueException
	 */
	#[Override] public function fromSql(array|int|string $value): Carbon
	{
		if (is_array($value))
			throw new InvalidStoredValueException($this, $value);

		if (is_int($value))
			// If it's an integer, we suppose that it's a unix timestamp.
			Carbon::createFromFormat("U", "$value");

		return Carbon::createFromFormat(static::SQL_FORMAT, $value);
	}
}
