<?php

namespace Nest\Types\Definitions\Integers;

use Nest\Types\Exceptions\InvalidSerializedValueException;
use Nest\Types\Exceptions\InvalidStoredValueException;
use Nest\Types\SerializableType;
use Nest\Types\SqlType;
use Nest\Types\Type;
use Override;

class IntType extends Type implements SerializableType, SqlType
{
	/**
	 * @inheritDoc
	 */
	#[Override] public function serialize(mixed $value): int
	{
		return intval($value);
	}

	/**
	 * @inheritDoc
	 * @throws InvalidSerializedValueException
	 */
	#[Override] public function deserialize(float|object|int|array|string $value): int
	{
		if (!is_int($value))
			throw new InvalidSerializedValueException($this, $value);

		return intval($value);
	}

	/**
	 * @inheritDoc
	 */
	#[Override] public function toSql(mixed $value): int
	{
		return intval($value);
	}

	/**
	 * @inheritDoc
	 * @throws InvalidStoredValueException
	 */
	#[Override] public function fromSql(array|int|string $value): int
	{
		if (is_array($value))
			throw new InvalidStoredValueException($this, $value);

		return intval($value);
	}
}
