<?php

namespace Nest\Types\Definitions;

use Nest\Types\Exceptions\InvalidSerializedValueException;
use Nest\Types\Exceptions\InvalidStoredValueException;
use Nest\Types\SerializableType;
use Nest\Types\SqlType;
use Nest\Types\Type;
use Override;

class StringType extends Type implements SerializableType, SqlType
{
	/**
	 * @inheritDoc
	 */
	#[Override] public function serialize(mixed $value): string
	{
		return (string) $value;
	}

	/**
	 * @inheritDoc
	 * @throws InvalidSerializedValueException
	 */
	#[Override] public function deserialize(float|object|int|array|string $value): string
	{
		if (!is_string($value))
			throw new InvalidSerializedValueException($this, $value);

		return $value;
	}

	/**
	 * @inheritDoc
	 */
	#[Override] public function toSql(mixed $value): string
	{
		return (string) $value;
	}

	/**
	 * @inheritDoc
	 * @throws InvalidStoredValueException
	 */
	#[Override] public function fromSql(array|int|string $value): string
	{
		if (is_array($value))
			throw new InvalidStoredValueException($this, $value);

		return (string) $value;
	}
}
